/*
 * kcmlisa.h
 *
 * Copyright (c) 2000-2002 Alexander Neundorf <neundorf@kde.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef KCMLISA_H
#define KCMLISA_H

#include <tdeconfig.h>
#include <tdecmodule.h>

class TQPushButton;
class TQCheckBox;
class TQSpinBox;
class TQPushButton;
class TDEProcess;
class KDialogBase;
class KRestrictedLine;
class KEditListBox;
class SetupWizard;

class LisaSettings:public TDECModule
{
   TQ_OBJECT
  
   public:
      LisaSettings(const TQString& config, TQWidget *parent=0);
      virtual ~LisaSettings() {};
      void load();
      void save();
   signals:
      void changed();
   protected slots:
      void slotChanged();
      void autoSetup();
      void saveDone(TDEProcess *);  // called after the tdesud returns (on save)
      void suggestSettings();
   protected:
      TDEConfig m_config;
      TQPushButton *m_autoSetup;
      TQCheckBox *m_useNmblookup;
      TQCheckBox *m_sendPings;
      KRestrictedLine *m_pingAddresses;
      KEditListBox *m_pingNames;
      KRestrictedLine *m_allowedAddresses;
      KRestrictedLine *m_broadcastNetwork;
      TQSpinBox *m_firstWait;
      TQCheckBox *m_secondScan;
      TQSpinBox *m_secondWait;
      TQSpinBox *m_updatePeriod;
      TQCheckBox *m_deliverUnnamedHosts;
      TQSpinBox *m_maxPingsAtOnce;
      TQPushButton* m_suggestSettings;
      TQPushButton *m_advancedSettingsButton;
      KDialogBase *m_lisaAdvancedDlg;

      SetupWizard *m_wizard;
   private:
      TQString m_tmpFilename;
      TQString m_configFilename;
      bool    m_changed;
};

#endif

